import { Widget } from '@lumino/widgets';
import checkSVGstr from '../style/icon/check.svg';
import { showMessage } from './ConfirmMessage';
import { Dialog, showDialog } from '@jupyterlab/apputils';
import { CodeSnippetContentsService } from './CodeSnippetContentsService';
/**
 * The class name added to file dialogs.
 */
const FILE_DIALOG_CLASS = 'jp-FileDialog';
/**
 * The class name added for the new name label in the rename dialog
 */
const INPUT_NEWNAME_TITLE_CLASS = 'jp-new-name-title';
/**
 * Save an input with a dialog. This is what actually displays everything.
 * Result.value is the value retrieved from .getValue(). ---> .getValue() returns an array of inputs.
 */
export function inputDialog(codeSnippet, code, idx) {
    return showDialog({
        title: 'Save Code Snippet',
        body: new InputHandler(),
        focusNodeSelector: 'input',
        buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Save' })]
    }).then((result) => {
        if (validateForm(result) === false) {
            return inputDialog(codeSnippet, code, idx); // This works but it wipes out all the data they entered previously...
        }
        if (!result.value) {
            return null;
        }
        else {
            if (idx === -1) {
                idx = codeSnippet.codeSnippetWidgetModel.snippets.length;
            }
            const newSnippet = {
                name: result.value[0].replace(' ', '').toLowerCase(),
                description: result.value[1],
                language: result.value[2],
                code: code,
                id: idx
            };
            const contentsService = CodeSnippetContentsService.getInstance();
            const currSnippets = codeSnippet.codeSnippetWidgetModel.snippets;
            for (const snippet of currSnippets) {
                if (snippet.name === newSnippet.name) {
                    // const oldPath = 'snippets/' + snippet.name + '.json';
                    const result = saveOverWriteFile(codeSnippet.codeSnippetWidgetModel, snippet, newSnippet);
                    result
                        .then(newSnippets => {
                        codeSnippet.renderCodeSnippetsSignal.emit(newSnippets);
                    })
                        .catch(_ => {
                        console.log('cancelling overwrite!');
                    });
                    return;
                }
            }
            createNewSnippet(codeSnippet, newSnippet, contentsService);
        }
    });
}
function createNewSnippet(codeSnippet, newSnippet, contentsService) {
    const request = contentsService.save('snippets/' + newSnippet.name + '.json', {
        type: 'file',
        format: 'text',
        content: JSON.stringify(newSnippet)
    });
    request.then(_ => {
        // add the new snippet to the snippet model
        codeSnippet.codeSnippetWidgetModel.addSnippet(newSnippet, newSnippet.id);
        codeSnippet.codeSnippetWidgetModel.updateSnippetContents();
        const newSnippets = codeSnippet.codeSnippetWidgetModel.snippets;
        codeSnippet.codeSnippets = newSnippets;
        codeSnippet.renderCodeSnippetsSignal.emit(newSnippets);
        showMessage({
            body: /*"Saved as Snippet"*/ new MessageHandler()
        });
    });
}
/**
 * Rename a file, asking for confirmation if it is overwriting another.
 */
async function saveOverWriteFile(codeSnippetWidgetModel, oldSnippet, newSnippet) {
    const newPath = 'snippets/' + newSnippet.name + '.json';
    return await shouldOverwrite(newPath).then(value => {
        if (value) {
            newSnippet.id = oldSnippet.id;
            codeSnippetWidgetModel.deleteSnippet(oldSnippet.id);
            codeSnippetWidgetModel.addSnippet(newSnippet, oldSnippet.id);
            codeSnippetWidgetModel.updateSnippetContents();
            return codeSnippetWidgetModel.snippets;
        }
        return Promise.reject('File not renamed');
    });
}
/**
 * Ask the user whether to overwrite a file.
 */
async function shouldOverwrite(path) {
    const options = {
        title: 'Overwrite code snippet?',
        body: `"${path}" already exists, overwrite?`,
        buttons: [Dialog.cancelButton(), Dialog.warnButton({ label: 'Overwrite' })]
    };
    return showDialog(options).then(result => {
        return Promise.resolve(result.button.accept);
    });
}
/**
 * Test whether a name is a valid file name
 *
 * Disallows "/", "\", and ":" in file names, as well as names with zero length.
 */
export function isValidFileName(name) {
    const validNameExp = /[/\\:]/;
    return name.length > 0 && !validNameExp.test(name);
}
/**
 * Test whether user typed in all required inputs.
 */
export function validateForm(input) {
    let status = true;
    let message = '';
    const name = input.value[0];
    const description = input.value[1];
    const language = input.value[2];
    if (name === '') {
        message += 'Name must be filled out\n';
        //alert("Description must be filled out");
        status = false;
    }
    if (description === '') {
        message += 'Description must be filled out\n';
        //alert("");
        status = false;
    }
    if (language === '') {
        message += 'Language must be filled out';
        //alert("Description ");
        status = false;
    }
    if (status === false) {
        alert(message);
    }
    return status;
}
/**
 * A widget used to get input data.
 */
class InputHandler extends Widget {
    /**
     * Construct a new "rename" dialog.
     * readonly inputNode: HTMLInputElement; <--- in Widget class
     */
    constructor() {
        super({ node: Private.createInputNode() });
        this.addClass(FILE_DIALOG_CLASS);
        console.log(this.node.getElementsByTagName('select')[0].value);
    }
    getValue() {
        const inputs = [];
        inputs.push(this.node.getElementsByTagName('input')[0].value, this.node.getElementsByTagName('input')[1].value, this.node.getElementsByTagName('select')[0].value);
        return inputs;
    }
}
class MessageHandler extends Widget {
    constructor() {
        super({ node: Private.createConfirmMessageNode() });
    }
}
// /**
//  * A method to create an input when the user selects the Other option.
//  */
// function createLanguageInput(form: HTMLFormElement): void {
//   const newInput = document.createElement('input');
//   newInput.id = 'alternate-user-input';
//   form.appendChild(newInput);
// }
// /**
//  * A method to remove an input when the user selects the Other option.
//  */
// function removeLanguageInput(form: HTMLFormElement): void {
//   const unusedInput = document.getElementById('alternate-user-input');
//   if (form.contains(unusedInput)) {
//     form.removeChild(unusedInput);
//   }
// }
/**
 * A namespace for private data.
 */
class Private {
    /**
     * Create the node for a rename handler. This is what's creating all of the elements to be displayed.
     */
    static createInputNode() {
        const body = document.createElement('form');
        const nameTitle = document.createElement('label');
        nameTitle.textContent = 'Snippet Name*';
        nameTitle.className = INPUT_NEWNAME_TITLE_CLASS;
        const name = document.createElement('input');
        const descriptionTitle = document.createElement('label');
        descriptionTitle.textContent = 'Description*';
        descriptionTitle.className = INPUT_NEWNAME_TITLE_CLASS;
        const description = document.createElement('input');
        description.required = true;
        const languageTitle = document.createElement('label');
        languageTitle.textContent = 'Language*';
        languageTitle.className = INPUT_NEWNAME_TITLE_CLASS;
        const language = document.createElement('select');
        const optionPython = document.createElement('option');
        optionPython.textContent = 'python';
        const optionR = document.createElement('option');
        optionR.textContent = 'R';
        const optionScala = document.createElement('option');
        optionScala.textContent = 'Scala';
        const optionOther = document.createElement('option');
        optionOther.textContent = 'Other';
        // optionOther.onclick = (): void => {
        //   createLanguageInput(body);
        // };
        // optionPython.onclick = (): void => {
        //   removeLanguageInput(body);
        // };
        // optionScala.onclick = (): void => {
        //   removeLanguageInput(body);
        // };
        // optionR.onclick = (): void => {
        //   removeLanguageInput(body);
        // };
        language.appendChild(optionPython);
        language.appendChild(optionR);
        language.appendChild(optionScala);
        language.appendChild(optionOther);
        // if (language.value === 'Other') {
        //   createLanguageInput(body);
        // } else {
        //   removeLanguageInput(body);
        // }
        body.appendChild(nameTitle);
        body.appendChild(name);
        body.appendChild(descriptionTitle);
        body.appendChild(description);
        body.appendChild(languageTitle);
        body.appendChild(language);
        return body;
    }
    static createConfirmMessageNode() {
        const body = document.createElement('div');
        body.innerHTML = checkSVGstr;
        const messageContainer = document.createElement('div');
        messageContainer.className = 'jp-confirm-text';
        const message = document.createElement('text');
        message.textContent = 'Saved as Snippet!';
        messageContainer.appendChild(message);
        body.append(messageContainer);
        return body;
    }
}
